@@ -6,6 +6,7 @@ from typing import TYPE_CHECKING
 
 import numpy as np
 
+from .._logging import logger
 from .._units import Quantity
 from .._units import units as plonk_units
 from ..utils.math import cross, norm
@@ -14,6 +15,7 @@ if TYPE_CHECKING:
     from ..snap.snap import SnapLike
 
 ORIGIN = (0, 0, 0) * plonk_units.au
+MOLECULAR_HYDROGEN_WEIGHT = 2.381
 
 # Derived quantities require some arrays already present
 array_requires = {
@@ -32,13 +34,15 @@ array_requires = {
     'kinetic_energy': ('mass', 'velocity'),
     'momentum': ('mass', 'velocity'),
     'polar_angle': ('position',),
-    'radial_distance': ('position',),
-    'radial_velocity': ('position', 'velocity'),
+    'radius_cylindrical': ('position',),
+    'radius_spherical': ('position',),
     'semi_major_axis': ('position', 'velocity'),
     'specific_angular_momentum': ('position', 'velocity'),
     'specific_kinetic_energy': 'velocity',
     'stokes_number': ('position', 'stopping_time'),
     'temperature': ('sound_speed',),
+    'velocity_radial_cylindrical': ('position', 'velocity'),
+    'velocity_radial_spherical': ('position', 'velocity'),
 }
 
 # Arrays which represent quantities with x-, y-, z-components in space
@@ -48,34 +52,8 @@ vector_arrays = ['angular_momentum', 'momentum', 'specific_angular_momentum']
 dust_arrays = ['dust_density', 'dust_fraction', 'dust_mass', 'stokes_number']
 
 
-def momentum(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
-    """Calculate the momentum.
-
-    Parameters
-    ----------
-    snap
-        The Snap object.
-    ignore_accreted : optional
-        Ignore accreted particles. Default is False.
-
-    Returns
-    -------
-    Quantity
-        The linear momentum on the particles.
-    """
-    if ignore_accreted:
-        h: Quantity = snap['smoothing_length']
-        mass: Quantity = snap['mass'][h > 0]
-        vel: Quantity = snap['velocity'][h > 0]
-    else:
-        mass = snap['mass']
-        vel = snap['velocity']
-
-    return mass[:, np.newaxis] * vel
-
-
 def angular_momentum(
-    snap: SnapLike, origin: Quantity = ORIGIN, ignore_accreted: bool = False
+    snap: SnapLike, origin: Quantity = None, ignore_accreted: bool = False
 ) -> Quantity:
     """Calculate the angular momentum.
 
@@ -104,30 +82,26 @@ def angular_momentum(
         pos = snap['position']
         vel = snap['velocity']
 
+    origin = snap.translation if snap.translation is not None else ORIGIN
     pos = pos - origin
 
     return mass[:, np.newaxis] * cross(pos, vel)
 
 
-def specific_angular_momentum(
-    snap: SnapLike, origin: Quantity = ORIGIN, ignore_accreted: bool = False
-) -> Quantity:
-    """Calculate the specific angular momentum.
+def angular_velocity(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
+    """Calculate the angular velocity.
 
     Parameters
     ----------
     snap
         The Snap object.
-    origin : optional
-        The origin around which to compute the angular momentum as a
-        Quantity like (x, y, z) * au. Default is (0, 0, 0).
     ignore_accreted : optional
         Ignore accreted particles. Default is False.
 
     Returns
     -------
     Quantity
-        The specific angular momentum on the particles.
+        The angular velocity on the particles.
     """
     if ignore_accreted:
         h: Quantity = snap['smoothing_length']
@@ -137,13 +111,16 @@ def specific_angular_momentum(
         pos = snap['position']
         vel = snap['velocity']
 
-    pos = pos - origin
+    x, y = pos[:, 0], pos[:, 1]
+    vx, vy = vel[:, 0], vel[:, 1]
 
-    return cross(pos, vel)
+    vphi = (x * vy - y * vx) / (x ** 2 + y ** 2)
+
+    return vphi
 
 
-def kinetic_energy(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
-    """Calculate the kinetic energy.
+def azimuthal_angle(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
+    """Calculate the azimuthal angle.
 
     Parameters
     ----------
@@ -155,71 +132,7 @@ def kinetic_energy(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
     Returns
     -------
     Quantity
-        The kinetic energy on the particles.
-    """
-    if ignore_accreted:
-        h: Quantity = snap['smoothing_length']
-        mass: Quantity = snap['mass'][h > 0]
-        vel: Quantity = snap['velocity'][h > 0]
-    else:
-        mass = snap['mass']
-        vel = snap['velocity']
-
-    return 1 / 2 * mass * norm(vel, axis=1) ** 2
-
-
-def specific_kinetic_energy(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
-    """Calculate the specific kinetic energy.
-
-    Parameters
-    ----------
-    snap
-        The Snap object.
-    ignore_accreted : optional
-        Ignore accreted particles. Default is False.
-
-    Returns
-    -------
-    Quantity
-        The specific kinetic energy on the particles.
-    """
-    if ignore_accreted:
-        h: Quantity = snap['smoothing_length']
-        vel: Quantity = snap['velocity'][h > 0]
-    else:
-        vel = snap['velocity']
-
-    return 1 / 2 * norm(vel, axis=1) ** 2
-
-
-def keplerian_frequency(
-    snap: SnapLike,
-    gravitational_parameter: Quantity,
-    origin: Quantity = ORIGIN,
-    ignore_accreted: bool = False,
-) -> Quantity:
-    """Calculate the Keplerian orbital frequency.
-
-    The Keplerian orbital frequency of particles around a mass specified
-    by gravitational parameter with an optional to specify the position
-    of the mass.
-
-    Parameters
-    ----------
-    snap
-        The Snap object.
-    gravitational_parameter
-        The gravitational parameter (mu = G M) as a Pint quantity.
-    origin : optional
-        The origin around which to compute the Keplerian frequency as a
-        Quantity like (x, y, z) * au. Default is (0, 0, 0).
-    ignore_accreted : optional
-        Ignore accreted particles. Default is False.
-
-    Returns
-    -------
-    Quantity
-        The Keplerian frequency on the particles.
+        The azimuthal angle on the particles.
     """
     if ignore_accreted:
         h: Quantity = snap['smoothing_length']
@@ -227,210 +140,12 @@ def keplerian_frequency(
     else:
         pos = snap['position']
 
-    pos = pos - origin
-
-    mu = gravitational_parameter
-
-    radius = norm(pos, axis=1)
-    return np.sqrt(mu / radius ** 3)
+    x, y = pos[:, 0], pos[:, 1]
+    return np.arctan2(y, x)
 
 
-def stokes_number(
-    snap: SnapLike,
-    gravitational_parameter: Quantity,
-    origin: Quantity = ORIGIN,
-    ignore_accreted: bool = False,
-) -> Quantity:
-    """Calculate the Stokes number.
-
-    Parameters
-    ----------
-    snap
-        The Snap object.
-    gravitational_parameter
-        The gravitational parameter (mu = G M) as a Pint quantity.
-    origin : optional
-        The origin around which to compute the Stokes number as a
-        Quantity like (x, y, z) * au. Default is (0, 0, 0).
-    ignore_accreted : optional
-        Ignore accreted particles. Default is False.
-
-    Returns
-    -------
-    Quantity
-        The Stokes number on the particles.
-    """
-    if ignore_accreted:
-        h: Quantity = snap['smoothing_length']
-        pos: Quantity = snap['position'][h > 0]
-        t_s: Quantity = snap['stopping_time'][h > 0]
-    else:
-        pos = snap['position']
-        t_s = snap['stopping_time']
-
-    pos = pos - origin
-
-    mu = gravitational_parameter
-
-    radius = norm(pos, axis=1)
-    Omega_k = np.sqrt(mu / radius ** 3)
-
-    Stokes = t_s * Omega_k[:, np.newaxis]
-    return Stokes
-
-
-def semi_major_axis(
-    snap: SnapLike,
-    gravitational_parameter: Quantity,
-    origin: Quantity = ORIGIN,
-    ignore_accreted: bool = False,
-) -> Quantity:
-    """Calculate the semi-major axis.
-
-    The semi-major axis of particles around a mass specified by
-    gravitational parameter with an optional to specify the position of
-    the mass.
-
-    Parameters
-    ----------
-    snap
-        The Snap object.
-    gravitational_parameter
-        The gravitational parameter (mu = G M) as a Pint quantity.
-    origin : optional
-        The origin around which to compute the semi-major axis as a
-        Quantity like (x, y, z) * au. Default is (0, 0, 0).
-    ignore_accreted : optional
-        Ignore accreted particles. Default is False.
-
-    Returns
-    -------
-    Quantity
-        The semi-major axis on the particles.
-    """
-    if ignore_accreted:
-        h: Quantity = snap['smoothing_length']
-        pos: Quantity = snap['position'][h > 0]
-        vel: Quantity = snap['velocity'][h > 0]
-    else:
-        pos = snap['position']
-        vel = snap['velocity']
-
-    pos = pos - origin
-
-    mu = gravitational_parameter
-
-    radius = norm(pos, axis=1)
-
-    _specific_angular_momentum = cross(pos, vel)
-    specific_angular_momentum_magnitude = norm(_specific_angular_momentum, axis=1)
-
-    _specific_kinetic_energy = 1 / 2 * norm(vel, axis=1) ** 2
-    specific_potential_energy = -mu / radius
-    specific_energy = _specific_kinetic_energy + specific_potential_energy
-
-    term = specific_energy * (specific_angular_momentum_magnitude / mu) ** 2
-
-    _eccentricity = np.sqrt(1 + 2 * term)
-
-    return specific_angular_momentum_magnitude ** 2 / (mu * (1 - _eccentricity ** 2))
-
-
-def eccentricity(
-    snap: SnapLike,
-    gravitational_parameter: Quantity,
-    origin: Quantity = ORIGIN,
-    ignore_accreted: bool = False,
-) -> Quantity:
-    """Calculate the eccentricity.
-
-    The eccentricity of particles around a mass specified by
-    gravitational parameter with an optional to specify the position of
-    the mass.
-
-    Parameters
-    ----------
-    snap
-        The Snap object.
-    gravitational_parameter
-        The gravitational parameter (mu = G M) as a Pint quantity.
-    origin : optional
-        The origin around which to compute the eccentricity as a
-        Quantity like (x, y, z) * au. Default is (0, 0, 0).
-    ignore_accreted : optional
-        Ignore accreted particles. Default is False.
-
-    Returns
-    -------
-    Quantity
-        The eccentricity on the particles.
-    """
-    if ignore_accreted:
-        h: Quantity = snap['smoothing_length']
-        pos: Quantity = snap['position'][h > 0]
-        vel: Quantity = snap['velocity'][h > 0]
-    else:
-        pos = snap['position']
-        vel = snap['velocity']
-
-    pos = pos - origin
-
-    mu = gravitational_parameter
-
-    radius = norm(pos, axis=1)
-
-    _specific_angular_momentum = cross(pos, vel)
-    specific_angular_momentum_magnitude = norm(_specific_angular_momentum, axis=1)
-
-    _specific_kinetic_energy = 1 / 2 * norm(vel, axis=1) ** 2
-    specific_potential_energy = -mu / radius
-    specific_energy = _specific_kinetic_energy + specific_potential_energy
-
-    term = specific_energy * (specific_angular_momentum_magnitude / mu) ** 2
-    return np.sqrt(1 + 2 * term)
-
-
-def inclination(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
-    """Calculate the inclination.
-
-    The inclination is calculated by taking the angle between the
-    angular momentum vector and the z-axis, with the angular momentum
-    calculated with respect to the center of mass.
-
-    Parameters
-    ----------
-    snap
-        The Snap object.
-    ignore_accreted : optional
-        Ignore accreted particles. Default is False.
-
-    Returns
-    -------
-    Quantity
-        The inclination on the particles.
-    """
-    if ignore_accreted:
-        h: Quantity = snap['smoothing_length']
-        mass: Quantity = snap['mass'][h > 0]
-        pos: Quantity = snap['position'][h > 0]
-        vel: Quantity = snap['velocity'][h > 0]
-    else:
-        mass = snap['mass']
-        pos = snap['position']
-        vel = snap['velocity']
-
-    origin = (mass[:, np.newaxis] * pos).sum(axis=0) / mass.sum()
-    pos = pos - origin
-
-    _specific_angular_momentum = cross(pos, vel)
-
-    return np.arccos(
-        _specific_angular_momentum[:, 2] / norm(_specific_angular_momentum, axis=1)
-    )
-
-
-def gas_fraction(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
-    """Calculate the gas fraction.
+def dust_density(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
+    """Calculate the dust density per species.
 
     For dust/gas mixtures this is from the dust fraction.
 
@@ -444,90 +159,28 @@ def gas_fraction(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
     Returns
     -------
     Quantity
-        The gas fraction on the particles.
+        The dust density per species on the particles.
     """
     if snap.properties['dust_method'] == 'dust/gas mixture':
         _dust_fraction: Quantity = snap['dust_fraction']
-        _gas_fraction = 1 - _dust_fraction.sum(axis=1)
 
     elif snap.properties['dust_method'] == 'dust as separate sets of particles':
-        particle_type = snap['type']
-        _gas_fraction = np.ones(len(snap))
-        _gas_fraction[particle_type != snap.particle_type['gas']] = 0
+        n_dust = len(snap.properties.get('grain_size', []))
+        sub_type = snap['sub_type']
+        type_mask = snap['type'] == snap.particle_type['dust']
+        _dust_fraction = np.zeros((len(snap), n_dust))
+        for idx in range(n_dust):
+            _dust_fraction[type_mask & (sub_type == idx + 1), idx] = 1
 
     else:
-        _gas_fraction = np.ones(len(snap))
-
-    if ignore_accreted:
-        h: Quantity = snap['smoothing_length']
-        return _gas_fraction[h > 0]
-    return _gas_fraction
-
-
-def gas_mass(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
-    """Calculate the gas mass.
-
-    For dust/gas mixtures this is from the dust fraction.
-
-    Parameters
-    ----------
-    snap
-        The Snap object.
-    ignore_accreted : optional
-        Ignore accreted particles. Default is False.
-
-    Returns
-    -------
-    Quantity
-        The gas mass on the particles.
-    """
-    if snap.properties['dust_method'] == 'dust/gas mixture':
-        _dust_fraction: Quantity = snap['dust_fraction']
-        _gas_fraction = 1 - _dust_fraction.sum(axis=1)
-
-    elif snap.properties['dust_method'] == 'dust as separate sets of particles':
-        particle_type = snap['type']
-        _gas_fraction = np.zeros(len(snap))
-        _gas_fraction[particle_type == snap.particle_type['gas']] = 1
-
-    mass: Quantity = snap['mass']
-    if ignore_accreted:
-        h: Quantity = snap['smoothing_length']
-        return (mass * _gas_fraction)[h > 0]
-    return mass * _gas_fraction
-
-
-def gas_density(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
-    """Calculate the gas density.
-
-    For dust/gas mixtures this is from the dust fraction.
-
-    Parameters
-    ----------
-    snap
-        The Snap object.
-    ignore_accreted : optional
-        Ignore accreted particles. Default is False.
-
-    Returns
-    -------
-    Quantity
-        The gas density on the particles.
-    """
-    if snap.properties['dust_method'] == 'dust/gas mixture':
-        _dust_fraction: Quantity = snap['dust_fraction']
-        _gas_fraction = 1 - _dust_fraction.sum(axis=1)
-
-    elif snap.properties['dust_method'] == 'dust as separate sets of particles':
-        particle_type = snap['type']
-        _gas_fraction = np.zeros(len(snap))
-        _gas_fraction[particle_type == snap.particle_type['gas']] = 1
+        raise ValueError('No dust available')
 
     density: Quantity = snap['density']
     if ignore_accreted:
         h: Quantity = snap['smoothing_length']
-        return (density * _gas_fraction)[h > 0]
-    return density * _gas_fraction
+        _dust_mass = density[:, np.newaxis] * _dust_fraction
+        return _dust_mass[h > 0]
+    return density[:, np.newaxis] * _dust_fraction
 
 
 def dust_fraction(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
@@ -606,8 +259,68 @@ def dust_mass(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
     return mass[:, np.newaxis] * _dust_fraction
 
 
-def dust_density(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
-    """Calculate the dust density per species.
+def eccentricity(
+    snap: SnapLike,
+    gravitational_parameter: Quantity = None,
+    origin: Quantity = None,
+    ignore_accreted: bool = False,
+) -> Quantity:
+    """Calculate the eccentricity.
+
+    The eccentricity of particles around a mass specified by
+    gravitational parameter with an optional to specify the position of
+    the mass.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    gravitational_parameter
+        The gravitational parameter (mu = G M) as a Pint quantity.
+    origin : optional
+        The origin around which to compute the eccentricity as a
+        Quantity like (x, y, z) * au. Default is (0, 0, 0).
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    Quantity
+        The eccentricity on the particles.
+    """
+    if ignore_accreted:
+        h: Quantity = snap['smoothing_length']
+        pos: Quantity = snap['position'][h > 0]
+        vel: Quantity = snap['velocity'][h > 0]
+    else:
+        pos = snap['position']
+        vel = snap['velocity']
+
+    origin = snap.translation if snap.translation is not None else ORIGIN
+    pos = pos - origin
+
+    mu = snap.properties.get('gravitational_parameter')
+    if mu is None:
+        raise ValueError(
+            'must pass in gravitational_parameter or '
+            'set on Snap with set_gravitational_parameter'
+        )
+
+    radius = norm(pos, axis=1)
+
+    _specific_angular_momentum = cross(pos, vel)
+    specific_angular_momentum_magnitude = norm(_specific_angular_momentum, axis=1)
+
+    _specific_kinetic_energy = 1 / 2 * norm(vel, axis=1) ** 2
+    specific_potential_energy = -mu / radius
+    specific_energy = _specific_kinetic_energy + specific_potential_energy
+
+    term = specific_energy * (specific_angular_momentum_magnitude / mu) ** 2
+    return np.sqrt(1 + 2 * term)
+
+
+def gas_density(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
+    """Calculate the gas density.
 
     For dust/gas mixtures this is from the dust fraction.
 
@@ -621,54 +334,159 @@ def dust_density(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
     Returns
     -------
     Quantity
-        The dust density per species on the particles.
+        The gas density on the particles.
     """
     if snap.properties['dust_method'] == 'dust/gas mixture':
         _dust_fraction: Quantity = snap['dust_fraction']
+        _gas_fraction = 1 - _dust_fraction.sum(axis=1)
 
     elif snap.properties['dust_method'] == 'dust as separate sets of particles':
-        n_dust = len(snap.properties.get('grain_size', []))
-        sub_type = snap['sub_type']
-        type_mask = snap['type'] == snap.particle_type['dust']
-        _dust_fraction = np.zeros((len(snap), n_dust))
-        for idx in range(n_dust):
-            _dust_fraction[type_mask & (sub_type == idx + 1), idx] = 1
-
-    else:
-        raise ValueError('No dust available')
+        particle_type = snap['type']
+        _gas_fraction = np.zeros(len(snap))
+        _gas_fraction[particle_type == snap.particle_type['gas']] = 1
 
     density: Quantity = snap['density']
     if ignore_accreted:
         h: Quantity = snap['smoothing_length']
-        _dust_mass = density[:, np.newaxis] * _dust_fraction
-        return _dust_mass[h > 0]
-    return density[:, np.newaxis] * _dust_fraction
+        return (density * _gas_fraction)[h > 0]
+    return density * _gas_fraction
 
 
-def radial_distance(
+def gas_fraction(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
+    """Calculate the gas fraction.
+
+    For dust/gas mixtures this is from the dust fraction.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    Quantity
+        The gas fraction on the particles.
+    """
+    if snap.properties['dust_method'] == 'dust/gas mixture':
+        _dust_fraction: Quantity = snap['dust_fraction']
+        _gas_fraction = 1 - _dust_fraction.sum(axis=1)
+
+    elif snap.properties['dust_method'] == 'dust as separate sets of particles':
+        particle_type = snap['type']
+        _gas_fraction = np.ones(len(snap))
+        _gas_fraction[particle_type != snap.particle_type['gas']] = 0
+
+    else:
+        _gas_fraction = np.ones(len(snap))
+
+    if ignore_accreted:
+        h: Quantity = snap['smoothing_length']
+        return _gas_fraction[h > 0]
+    return _gas_fraction
+
+
+def gas_mass(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
+    """Calculate the gas mass.
+
+    For dust/gas mixtures this is from the dust fraction.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    Quantity
+        The gas mass on the particles.
+    """
+    if snap.properties['dust_method'] == 'dust/gas mixture':
+        _dust_fraction: Quantity = snap['dust_fraction']
+        _gas_fraction = 1 - _dust_fraction.sum(axis=1)
+
+    elif snap.properties['dust_method'] == 'dust as separate sets of particles':
+        particle_type = snap['type']
+        _gas_fraction = np.zeros(len(snap))
+        _gas_fraction[particle_type == snap.particle_type['gas']] = 1
+
+    mass: Quantity = snap['mass']
+    if ignore_accreted:
+        h: Quantity = snap['smoothing_length']
+        return (mass * _gas_fraction)[h > 0]
+    return mass * _gas_fraction
+
+
+def inclination(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
+    """Calculate the inclination.
+
+    The inclination is calculated by taking the angle between the
+    angular momentum vector and the z-axis, with the angular momentum
+    calculated with respect to the center of mass.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    Quantity
+        The inclination on the particles.
+    """
+    if ignore_accreted:
+        h: Quantity = snap['smoothing_length']
+        mass: Quantity = snap['mass'][h > 0]
+        pos: Quantity = snap['position'][h > 0]
+        vel: Quantity = snap['velocity'][h > 0]
+    else:
+        mass = snap['mass']
+        pos = snap['position']
+        vel = snap['velocity']
+
+    origin = (mass[:, np.newaxis] * pos).sum(axis=0) / mass.sum()
+    pos = pos - origin
+
+    _specific_angular_momentum = cross(pos, vel)
+
+    return np.arccos(
+        _specific_angular_momentum[:, 2] / norm(_specific_angular_momentum, axis=1)
+    )
+
+
+def keplerian_frequency(
     snap: SnapLike,
-    coordinates: str = 'cylindrical',
-    origin: Quantity = ORIGIN,
+    gravitational_parameter: Quantity = None,
+    origin: Quantity = None,
     ignore_accreted: bool = False,
 ) -> Quantity:
-    """Calculate the radial distance.
+    """Calculate the Keplerian orbital frequency.
 
-    Can compute the radial distance in cylindrical and spherical
-    coordinates.
+    The Keplerian orbital frequency of particles around a mass specified
+    by gravitational parameter with an optional to specify the position
+    of the mass.
 
     Parameters
     ----------
     snap
         The Snap object.
-    coordinates : optional
-        Either 'cylindrical' or 'spherical'. Default is 'cylindrical'.
+    gravitational_parameter
+        The gravitational parameter (mu = G M) as a Pint quantity.
+    origin : optional
+        The origin around which to compute the Keplerian frequency as a
+        Quantity like (x, y, z) * au. Default is (0, 0, 0).
     ignore_accreted : optional
         Ignore accreted particles. Default is False.
 
     Returns
     -------
     Quantity
-        The radial distance on the particles.
+        The Keplerian frequency on the particles.
     """
     if ignore_accreted:
         h: Quantity = snap['smoothing_length']
@@ -676,18 +494,22 @@ def radial_distance(
     else:
         pos = snap['position']
 
+    origin = snap.translation if snap.translation is not None else ORIGIN
     pos = pos - origin
-    x, y, z = pos[:, 0], pos[:, 1], pos[:, 2]
 
-    if coordinates == 'cylindrical':
-        return np.sqrt(x ** 2 + y ** 2)
-    if coordinates == 'spherical':
-        return np.sqrt(x ** 2 + y ** 2 + z ** 2)
-    raise ValueError('Cannot determine coordinates')
+    mu = snap.properties.get('gravitational_parameter')
+    if mu is None:
+        raise ValueError(
+            'must pass in gravitational_parameter or '
+            'set on Snap with set_gravitational_parameter'
+        )
+
+    radius = norm(pos, axis=1)
+    return np.sqrt(mu / radius ** 3)
 
 
-def azimuthal_angle(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
-    """Calculate the azimuthal angle.
+def kinetic_energy(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
+    """Calculate the kinetic energy.
 
     Parameters
     ----------
@@ -699,16 +521,43 @@ def azimuthal_angle(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
     Returns
     -------
     Quantity
-        The azimuthal angle on the particles.
+        The kinetic energy on the particles.
     """
     if ignore_accreted:
         h: Quantity = snap['smoothing_length']
-        pos: Quantity = snap['position'][h > 0]
+        mass: Quantity = snap['mass'][h > 0]
+        vel: Quantity = snap['velocity'][h > 0]
     else:
-        pos = snap['position']
+        mass = snap['mass']
+        vel = snap['velocity']
 
-    x, y = pos[:, 0], pos[:, 1]
-    return np.arctan2(y, x)
+    return 1 / 2 * mass * norm(vel, axis=1) ** 2
+
+
+def momentum(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
+    """Calculate the momentum.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    Quantity
+        The linear momentum on the particles.
+    """
+    if ignore_accreted:
+        h: Quantity = snap['smoothing_length']
+        mass: Quantity = snap['mass'][h > 0]
+        vel: Quantity = snap['velocity'][h > 0]
+    else:
+        mass = snap['mass']
+        vel = snap['velocity']
+
+    return mass[:, np.newaxis] * vel
 
 
 def polar_angle(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
@@ -737,51 +586,10 @@ def polar_angle(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
     return np.arccos(z / r)
 
 
-def radial_velocity(
-    snap: SnapLike, coordinates: str = 'cylindrical', ignore_accreted: bool = False
+def radius_cylindrical(
+    snap: SnapLike, origin: Quantity = None, ignore_accreted: bool = False,
 ) -> Quantity:
-    """Calculate the radial velocity.
-
-    Can compute the radial velocity in cylindrical and spherical
-    coordinates.
-
-    Parameters
-    ----------
-    snap
-        The Snap object.
-    coordinates : optional
-        Either 'cylindrical' or 'spherical'. Default is 'cylindrical'.
-    ignore_accreted : optional
-        Ignore accreted particles. Default is False.
-
-    Returns
-    -------
-    Quantity
-        The radial velocity on the particles.
-    """
-    if ignore_accreted:
-        h: Quantity = snap['smoothing_length']
-        pos: Quantity = snap['position'][h > 0]
-        vel: Quantity = snap['velocity'][h > 0]
-    else:
-        pos = snap['position']
-        vel = snap['velocity']
-
-    x, y, z = pos[:, 0], pos[:, 1], pos[:, 2]
-    vx, vy, vz = vel[:, 0], vel[:, 1], vel[:, 2]
-
-    if coordinates == 'cylindrical':
-        vr = (x * vx + y * vy) / np.sqrt(x ** 2 + y ** 2)
-    elif coordinates == 'spherical':
-        vr = (x * vx + y * vy + z * vz) / np.sqrt(x ** 2 + y ** 2 + z ** 2)
-    else:
-        raise ValueError('Cannot determine coordinates')
-
-    return vr
-
-
-def angular_velocity(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
-    """Calculate the angular velocity.
+    """Calculate the cylindrical radial distance.
 
     Parameters
     ----------
@@ -793,26 +601,224 @@ def angular_velocity(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
     Returns
     -------
     Quantity
-        The angular velocity on the particles.
+        The radial distance on the particles.
     """
     if ignore_accreted:
         h: Quantity = snap['smoothing_length']
         pos: Quantity = snap['position'][h > 0]
-        vel: Quantity = snap['velocity'][h > 0]
     else:
         pos = snap['position']
-        vel = snap['velocity']
 
+    origin = snap.translation if snap.translation is not None else ORIGIN
+    pos = pos - origin
     x, y = pos[:, 0], pos[:, 1]
-    vx, vy = vel[:, 0], vel[:, 1]
 
-    vphi = (x * vy - y * vx) / (x ** 2 + y ** 2)
+    return np.sqrt(x ** 2 + y ** 2)
 
-    return vphi
+
+def radius_spherical(
+    snap: SnapLike, origin: Quantity = None, ignore_accreted: bool = False,
+) -> Quantity:
+    """Calculate the spherical radial distance.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    Quantity
+        The radial distance on the particles.
+    """
+    if ignore_accreted:
+        h: Quantity = snap['smoothing_length']
+        pos: Quantity = snap['position'][h > 0]
+    else:
+        pos = snap['position']
+
+    origin = snap.translation if snap.translation is not None else ORIGIN
+    pos = pos - origin
+    x, y, z = pos[:, 0], pos[:, 1], pos[:, 2]
+
+    return np.sqrt(x ** 2 + y ** 2 + z ** 2)
+
+
+def semi_major_axis(
+    snap: SnapLike,
+    gravitational_parameter: Quantity = None,
+    origin: Quantity = None,
+    ignore_accreted: bool = False,
+) -> Quantity:
+    """Calculate the semi-major axis.
+
+    The semi-major axis of particles around a mass specified by
+    gravitational parameter with an optional to specify the position of
+    the mass.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    gravitational_parameter
+        The gravitational parameter (mu = G M) as a Pint quantity.
+    origin : optional
+        The origin around which to compute the semi-major axis as a
+        Quantity like (x, y, z) * au. Default is (0, 0, 0).
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    Quantity
+        The semi-major axis on the particles.
+    """
+    if ignore_accreted:
+        h: Quantity = snap['smoothing_length']
+        pos: Quantity = snap['position'][h > 0]
+        vel: Quantity = snap['velocity'][h > 0]
+    else:
+        pos = snap['position']
+        vel = snap['velocity']
+
+    origin = snap.translation if snap.translation is not None else ORIGIN
+    pos = pos - origin
+
+    mu = snap.properties.get('gravitational_parameter')
+    if mu is None:
+        raise ValueError(
+            'must pass in gravitational_parameter or '
+            'set on Snap with set_gravitational_parameter'
+        )
+
+    radius = norm(pos, axis=1)
+
+    _specific_angular_momentum = cross(pos, vel)
+    specific_angular_momentum_magnitude = norm(_specific_angular_momentum, axis=1)
+
+    _specific_kinetic_energy = 1 / 2 * norm(vel, axis=1) ** 2
+    specific_potential_energy = -mu / radius
+    specific_energy = _specific_kinetic_energy + specific_potential_energy
+
+    term = specific_energy * (specific_angular_momentum_magnitude / mu) ** 2
+
+    _eccentricity = np.sqrt(1 + 2 * term)
+
+    return specific_angular_momentum_magnitude ** 2 / (mu * (1 - _eccentricity ** 2))
+
+
+def specific_angular_momentum(
+    snap: SnapLike, origin: Quantity = None, ignore_accreted: bool = False
+) -> Quantity:
+    """Calculate the specific angular momentum.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    origin : optional
+        The origin around which to compute the angular momentum as a
+        Quantity like (x, y, z) * au. Default is (0, 0, 0).
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    Quantity
+        The specific angular momentum on the particles.
+    """
+    if ignore_accreted:
+        h: Quantity = snap['smoothing_length']
+        pos: Quantity = snap['position'][h > 0]
+        vel: Quantity = snap['velocity'][h > 0]
+    else:
+        pos = snap['position']
+        vel = snap['velocity']
+
+    origin = snap.translation if snap.translation is not None else ORIGIN
+    pos = pos - origin
+
+    return cross(pos, vel)
+
+
+def specific_kinetic_energy(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
+    """Calculate the specific kinetic energy.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    Quantity
+        The specific kinetic energy on the particles.
+    """
+    if ignore_accreted:
+        h: Quantity = snap['smoothing_length']
+        vel: Quantity = snap['velocity'][h > 0]
+    else:
+        vel = snap['velocity']
+
+    return 1 / 2 * norm(vel, axis=1) ** 2
+
+
+def stokes_number(
+    snap: SnapLike,
+    gravitational_parameter: Quantity = None,
+    origin: Quantity = None,
+    ignore_accreted: bool = False,
+) -> Quantity:
+    """Calculate the Stokes number.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    gravitational_parameter
+        The gravitational parameter (mu = G M) as a Pint quantity.
+    origin : optional
+        The origin around which to compute the Stokes number as a
+        Quantity like (x, y, z) * au. Default is (0, 0, 0).
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    Quantity
+        The Stokes number on the particles.
+    """
+    if ignore_accreted:
+        h: Quantity = snap['smoothing_length']
+        pos: Quantity = snap['position'][h > 0]
+        t_s: Quantity = snap['stopping_time'][h > 0]
+    else:
+        pos = snap['position']
+        t_s = snap['stopping_time']
+
+    origin = snap.translation if snap.translation is not None else ORIGIN
+    pos = pos - origin
+
+    mu = snap.properties.get('gravitational_parameter')
+    if mu is None:
+        raise ValueError(
+            'must pass in gravitational_parameter or '
+            'set on Snap with set_gravitational_parameter'
+        )
+
+    radius = norm(pos, axis=1)
+    Omega_k = np.sqrt(mu / radius ** 3)
+
+    Stokes = t_s * Omega_k[:, np.newaxis]
+    return Stokes
 
 
 def temperature(
-    snap: SnapLike, molecular_weight: float = 2.381, ignore_accreted: bool = False
+    snap: SnapLike, molecular_weight: float = None, ignore_accreted: bool = False
 ) -> Quantity:
     """Calculate the gas temperature.
 
@@ -839,7 +845,77 @@ def temperature(
 
     gamma = snap.properties['adiabatic_index']
 
+    molecular_weight = snap.properties.get('molecular_weight')
+    if molecular_weight is None:
+        logger.warning(
+            'molecular_weight not set, assuming 2.381 for molecular hydrogen\n'
+            'use set_molecular_weight to set on Snap'
+        )
+        molecular_weight = MOLECULAR_HYDROGEN_WEIGHT
+
     molecular_weight = molecular_weight * plonk_units('gram / mole')
     specific_gas_constant = (plonk_units.R / molecular_weight).to_base_units()
 
     return cs ** 2 / (gamma * specific_gas_constant)
+
+
+def velocity_radial_cylindrical(
+    snap: SnapLike, ignore_accreted: bool = False
+) -> Quantity:
+    """Calculate the cylindrical radial velocity.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    Quantity
+        The radial velocity on the particles.
+    """
+    if ignore_accreted:
+        h: Quantity = snap['smoothing_length']
+        pos: Quantity = snap['position'][h > 0]
+        vel: Quantity = snap['velocity'][h > 0]
+    else:
+        pos = snap['position']
+        vel = snap['velocity']
+
+    x, y = pos[:, 0], pos[:, 1]
+    vx, vy = vel[:, 0], vel[:, 1]
+
+    return (x * vx + y * vy) / np.sqrt(x ** 2 + y ** 2)
+
+
+def velocity_radial_spherical(
+    snap: SnapLike, ignore_accreted: bool = False
+) -> Quantity:
+    """Calculate the spherical radial velocity.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+    ignore_accreted : optional
+        Ignore accreted particles. Default is False.
+
+    Returns
+    -------
+    Quantity
+        The radial velocity on the particles.
+    """
+    if ignore_accreted:
+        h: Quantity = snap['smoothing_length']
+        pos: Quantity = snap['position'][h > 0]
+        vel: Quantity = snap['velocity'][h > 0]
+    else:
+        pos = snap['position']
+        vel = snap['velocity']
+
+    x, y, z = pos[:, 0], pos[:, 1], pos[:, 2]
+    vx, vy, vz = vel[:, 0], vel[:, 1], vel[:, 2]
+
+    return (x * vx + y * vy + z * vz) / np.sqrt(x ** 2 + y ** 2 + z ** 2)
