@@ -5,7 +5,7 @@
 from __future__ import annotations
 
 from bisect import bisect
-from functools import partial
+from copy import copy
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Tuple, Union
 
 import matplotlib.pyplot as plt
@@ -20,6 +20,7 @@ from .._units import Quantity
 from .._units import units as plonk_units
 from ..utils.math import average
 from ..utils.utils import is_documented_by, pretty_array_name
+from ._profiles import extra_profiles
 
 if TYPE_CHECKING:
     from ..snap.snap import SnapLike
@@ -48,11 +49,11 @@ class Profile:
         Cartesian along the x-axis. Default is 2.
     cmin : optional
         The minimum coordinate for binning. Can be a string, e.g.
-        '10 au', or a quantity with units, e.g. plonk.units['10 au'].
+        '10 au', or a quantity with units, e.g. plonk.units('10 au').
         Defaults to minimum on the particles.
     cmax : optional
         The maximum coordinate for binning. Can be a string, e.g.
-        '10 au', or a quantity with units, e.g. plonk.units['10 au'].
+        '10 au', or a quantity with units, e.g. plonk.units('10 au').
         Defaults to the 99 percentile distance.
     n_bins : optional
         The number of radial bins. Default is 100.
@@ -88,9 +89,9 @@ class Profile:
 
     >>> prof['aspect_ratio'] = prof['scale_height'] / prof['radius']
 
-    Alternatively use the profile_property decorator.
+    Alternatively use the add_profile decorator.
 
-    >>> @prof.profile_property
+    >>> @prof.add_profile
     ... def mass(prof):
     ...     M = prof.snap['mass']
     ...     return prof.particles_to_binned_quantity('sum', M)
@@ -106,26 +107,6 @@ class Profile:
     >>> prof.plot('radius', 'surface_density', units=units)
     """
 
-    _profile_functions: Dict[str, Callable] = {}
-
-    @staticmethod
-    def profile_property(fn: Callable) -> Callable:
-        """Decorate function to add profile to Profile.
-
-        Parameters
-        ----------
-        fn
-            A function that returns the profile as an array. The name of
-            the function is the string with which to reference the array.
-
-        Returns
-        -------
-        Callable
-            The function which returns the array.
-        """
-        Profile._profile_functions[fn.__name__] = fn
-        return fn
-
     def __init__(
         self,
         snap: SnapLike,
@@ -146,6 +127,8 @@ class Profile:
         self.properties: Dict[str, Any] = {}
 
         self._profiles: Dict[str, Quantity] = {}
+        self._profile_functions: Dict[str, Callable] = {}
+        self._default_units = copy(self.snap._default_units)
 
         self._weights = self.snap['mass']
         self._mask = self._setup_particle_mask(ignore_accreted)
@@ -165,14 +148,33 @@ class Profile:
         self._profiles[self._coordinate] = self.bin_centers
         self._profiles['number'] = np.histogram(
             self._x.magnitude, self.bin_edges.magnitude
-        )[0]
+        )[0] * plonk_units('dimensionless')
 
         try:
             num_separate_dust = len(self.snap.num_particles_of_type['dust'])
         except KeyError:
             num_separate_dust = 0
         num_mixture_dust_species = self.snap.num_dust_species - num_separate_dust
-        _generate_profiles(num_mixture_dust_species)
+
+        # Add pre-defined profiles
+        extra_profiles(self, num_mixture_dust_species)
+
+    def add_profile(self, fn: Callable) -> Callable:
+        """Decorate function to add profile to Profile.
+
+        Parameters
+        ----------
+        fn
+            A function that returns the profile as an array. The name of
+            the function is the string with which to reference the array.
+
+        Returns
+        -------
+        Callable
+            The function which returns the array.
+        """
+        self._profile_functions[fn.__name__] = fn
+        return fn
 
     def _setup_particle_mask(self, ignore_accreted: bool) -> ndarray:
         if ignore_accreted is False:
@@ -204,14 +206,12 @@ class Profile:
             rmin = Quantity(cmin)
             if not rmin.dimensionality == Quantity('cm').dimensionality:
                 raise ValueError('must specify cmin units, e.g. cmin="10 au"')
-            rmin = rmin.to_base_units()
         if cmax is None:
             rmax = np.percentile(self._x.magnitude, 99, axis=0) * self._x.units
         else:
             rmax = Quantity(cmax)
             if not rmax.dimensionality == Quantity('cm').dimensionality:
                 raise ValueError('must specify cmin units, e.g. cmax="100 au"')
-            rmax = rmax.to_base_units()
 
         return rmin, rmax
 
@@ -343,6 +343,37 @@ class Profile:
         snap_arrays = _1d_arrays(list(self.snap.available_arrays(verbose=True)))
         return sorted(set(loaded + available + snap_arrays))
 
+    @property
+    def default_units(self) -> Dict[str, Any]:
+        """Profile default units."""
+        return {
+            key: self._default_units[key] for key in sorted(self._default_units.keys())
+        }
+
+    def set_units(self, **kwargs) -> Profile:
+        """Set default unit for profiles.
+
+        Parameters
+        ----------
+        kwargs
+            Keyword arguments with keys as the profile name, e.g.
+            'pressure', and with values as the unit as a string, e.g.
+            'pascal'.
+
+        Examples
+        --------
+        Set multiple default units.
+
+        >>> profile.set_units(pressure='pascal', density='g/cm^3')
+        """
+        for key, val in kwargs.items():
+            defaults = list(self.default_units) + list(self.snap.default_units)
+            if key not in defaults:
+                logger.info(f'adding profile {key} to default_units dict')
+            self._default_units[key] = val
+
+        return self
+
     def base_array_name(self, name: str) -> str:
         """Get the base array name from a string.
 
@@ -372,7 +403,7 @@ class Profile:
         x: str,
         y: Union[str, List[str]],
         units: Dict[str, Union[str, List[str]]] = None,
-        std_dev_shading: bool = False,
+        std: str = None,
         ax: Any = None,
         ax_kwargs={},
         **kwargs,
@@ -391,8 +422,9 @@ class Profile:
             quantities such as 'position', 'density', 'velocity', and so on.
             The values are strings representing units, e.g. 'g/cm^3' for
             density.
-        std_dev_shading : optional
-            Add shading for standard deviation of profile.
+        std: optional
+            Add standard deviation on profile. Can be 'shading' or
+            'errorbar'.
         ax : optional
             A matplotlib Axes object to plot to.
         ax_kwargs
@@ -405,69 +437,34 @@ class Profile:
         ax
             The matplotlib Axes object.
         """
-        if isinstance(y, str):
-            y = [y]
-
-        if units is None:
-            x_unit, y_unit = None, None
-        else:
-            x_unit = units.get(self.base_array_name(x), str(self[x].units))
-            y_unit = [
-                units.get(self.base_array_name(_y), str(self[_y].units)) for _y in y
-            ]
-        if y_unit is not None:
-            if isinstance(y_unit, str):
-                y_unit = [y_unit for _ in range(len(y))]
-            if len(y) != len(y_unit):
-                raise ValueError('Length of y does not match length of y_unit')
-
-        _x = self[x]
-        if x_unit is not None:
-            _x = _x.to(x_unit)
+        if std is not None:
+            if std not in ('shading', 'errorbar'):
+                raise ValueError('std must be "shading" or "errorbar"')
 
         if ax is None:
             _, ax = plt.subplots()
 
-        xname = pretty_array_name(x)
-        ax.set_xlabel(f'{xname} [{_x.units:~P}]')
-        _x = _x.magnitude
+        if isinstance(y, str):
+            y = [y]
 
-        for idx, yi in enumerate(y):
-            _y = self[yi]
-            if std_dev_shading:
-                if yi.split('_')[-1] in _aggregations:
-                    _yi = '_'.join(yi.split('_')[:-1])
-                else:
-                    _yi = yi
-                try:
-                    _y_std = self[_yi + '_std']
-                except ValueError:
-                    logger.warning('Cannot calculate standard deviation')
-                if self.aggregation != 'mean':
-                    _y_mean = self[_yi + '_mean']
-                else:
-                    _y_mean = _y
-            if y_unit is not None:
-                _y = _y.to(y_unit[idx])
-                if std_dev_shading:
-                    _y_std = _y_std.to(y_unit[idx])
-                    _y_mean = _y_mean.to(y_unit[idx])
-            yname = pretty_array_name(yi)
-            label = f'{yname} [{_y.units:~P}]'
-            _y = _y.magnitude
-            if std_dev_shading:
-                _y_std = _y_std.magnitude
-                _y_mean = _y_mean.magnitude
+        x_unit = _get_unit(self, x, units)
+        y_unit = [_get_unit(self, yi, units) for yi in y]
+
+        xdata = self[x].to(x_unit)
+
+        xlabel = pretty_array_name(x)
+        ax.set_xlabel(f'{xlabel} [{xdata.units:~P}]')
+
+        for idx, yname in enumerate(y):
+            ydata = self[yname].to(y_unit[idx])
+            ylabel = pretty_array_name(yname)
+            label = f'{ylabel} [{ydata.units:~P}]'
             if kwargs.get('label') is None:
-                lines = ax.plot(_x, _y, label=label, **kwargs)
+                ax.plot(xdata.magnitude, ydata.magnitude, label=label, **kwargs)
             else:
-                lines = ax.plot(_x, _y, **kwargs)
-            if std_dev_shading:
-                color = lines[0].get_color()
-                ax.fill_between(
-                    _x, _y_mean - _y_std, _y_mean + _y_std, color=color, alpha=0.2,
-                )
-
+                ax.plot(xdata.magnitude, ydata.magnitude, **kwargs)
+            if std:
+                _std_plot(self, xdata, ydata, yname, y_unit[idx], std, ax)
         ax.legend()
         ax.set(**ax_kwargs)
 
@@ -540,7 +537,7 @@ class Profile:
                 try:
                     _units.append(self[column].units)
                 except AttributeError:
-                    _units.append(plonk_units['dimensionless'])
+                    _units.append(plonk_units('dimensionless'))
         else:
             _units = list()
             for unit in units:
@@ -608,98 +605,6 @@ class Profile:
         return binned_quantity
 
 
-def _generate_profiles(num_mixture_dust_species: int = 0):
-    @Profile.profile_property
-    def mass(prof) -> Quantity:
-        """Mass profile."""
-        M = prof.snap['mass']
-        return prof.particles_to_binned_quantity('sum', M)
-
-    @Profile.profile_property
-    def surface_density(prof) -> Quantity:
-        """Surface density profile.
-
-        Units are [mass / length ** ndim], which depends on ndim of profile.
-        """
-        return prof['mass'] / prof['size']
-
-    @Profile.profile_property
-    def scale_height(prof) -> Quantity:
-        """Scale height profile."""
-        z = prof.snap['z']
-        return prof.particles_to_binned_quantity('std', z)
-
-    @Profile.profile_property
-    def aspect_ratio(prof) -> Quantity:
-        """Aspect ratio profile."""
-        H = prof['scale_height']
-        R = prof['radius']
-        return H / R
-
-    @Profile.profile_property
-    def angular_momentum_theta(prof) -> Quantity:
-        """Angle between specific angular momentum and xy-plane."""
-        angular_momentum_z = prof['angular_momentum_z']
-        angular_momentum_magnitude = prof['angular_momentum_mag']
-
-        return np.arccos(angular_momentum_z / angular_momentum_magnitude)
-
-    @Profile.profile_property
-    def angular_momentum_phi(prof) -> Quantity:
-        """Angle between specific angular momentum and x-axis in xy-plane."""
-        angular_momentum_x = prof['angular_momentum_x']
-        angular_momentum_y = prof['angular_momentum_y']
-        return np.arctan2(angular_momentum_y, angular_momentum_x)
-
-    @Profile.profile_property
-    def toomre_Q(prof) -> Quantity:
-        """Toomre Q parameter."""
-        G = (1 * plonk_units.newtonian_constant_of_gravitation).to_base_units()
-        return (
-            prof['sound_speed']
-            * prof['keplerian_frequency']
-            / (np.pi * G * prof['surface_density'])
-        )
-
-    if num_mixture_dust_species > 0:
-
-        @Profile.profile_property
-        def gas_mass(prof) -> Quantity:
-            """Gas mass profile."""
-            M = prof.snap['gas_mass']
-            return prof.particles_to_binned_quantity('sum', M)
-
-        @Profile.profile_property
-        def gas_surface_density(prof) -> Quantity:
-            """Gas surface density profile.
-
-            Units are [mass / length ** ndim], which depends on ndim of profile.
-            """
-            return prof['gas_mass'] / prof['size']
-
-        for idx in range(num_mixture_dust_species):
-
-            def dust_mass(idx, prof) -> Quantity:
-                """Dust mass profile."""
-                M = prof.snap[f'dust_mass_{idx+1:03}']
-                return prof.particles_to_binned_quantity('sum', M)
-
-            def dust_surface_density(idx, prof) -> Quantity:
-                """Dust surface density profile.
-
-                Units are [mass / length ** ndim], which depends on ndim of profile.
-                """
-                return prof[f'dust_mass_{idx+1:03}'] / prof['size']
-
-            Profile._profile_functions[f'dust_mass_{idx+1:03}'] = partial(
-                dust_mass, idx
-            )
-
-            Profile._profile_functions[f'dust_surface_density_{idx+1:03}'] = partial(
-                dust_surface_density, idx
-            )
-
-
 @is_documented_by(Profile)
 def load_profile(
     snap: SnapLike,
@@ -750,3 +655,43 @@ def _1d_arrays(arrays: list) -> list:
         else:
             _arrays.append(array)
     return _arrays
+
+
+def _get_unit(profile, name, units):
+    if name is None:
+        return None
+    base_name = profile.base_array_name(name)
+    if units is not None:
+        if name in units:
+            return 1 * plonk_units(units[name])
+        if base_name in units:
+            return 1 * plonk_units(units[base_name])
+    if base_name in profile.default_units:
+        return 1 * plonk_units(profile.default_units[base_name])
+    return 1 * profile[base_name].units
+
+
+def _std_plot(profile, xdata, ydata, yname, yunit, std, ax):
+    if yname.split('_')[-1] in _aggregations:
+        _yname = '_'.join(yname.split('_')[:-1])
+    else:
+        _yname = yname
+    try:
+        y_std = profile[_yname + '_std']
+    except ValueError:
+        logger.warning('Cannot calculate standard deviation')
+        return
+    if profile.aggregation in ('std', 'sum'):
+        y_mean = profile[_yname + '_mean']
+    else:
+        y_mean = ydata
+    y_std = y_std.to(yunit).magnitude
+    y_mean = y_mean.to(yunit).magnitude
+    color = ax.lines[0].get_color()
+    xdata = xdata.magnitude
+    if std == 'shading':
+        ax.fill_between(
+            xdata, y_mean - y_std, y_mean + y_std, color=color, alpha=0.2,
+        )
+    elif std == 'errorbar':
+        ax.errorbar(xdata, y_mean, yerr=y_std, linestyle='', color=color, alpha=0.5)
