@@ -1,14 +1,15 @@
-"""The snap sub-package.
+"""Snapshot data access and manipulation.
 
 It contains Plonk the implementation of smoothed particle hydrodynamics
 snapshot file.
 """
 
 from .readers import load_snap
-from .snap import Snap, SnapLike, SubSnap
+from .snap import Sinks, Snap, SnapLike, SubSnap
 from .utils import gravitational_constant_in_code_units
 
 __all__ = [
+    'Sinks',
     'Snap',
     'SnapLike',
     'SubSnap',
