@@ -8,75 +8,48 @@ from typing import Callable, Dict, List, Union
 import h5py
 import numpy as np
 
+from ..._config import load_config
 from ..._logging import logger
-from ..._units import Quantity, generate_array_units_dict, generate_code_units_dict
+from ..._units import Quantity, array_units, generate_array_code_units
 from ..._units import units as plonk_units
 from ..extra import extra_quantities
 from ..snap import Snap
+from ..utils import add_aliases
 
 igas, iboundary, istar, idarkmatter, ibulge = 1, 3, 4, 5, 6
 bignumber = 1e29
 
-# Dictionary to convert Phantom HDF5 particle array names to Plonk names
-particle_array_name_map = {
-    'abundance': 'abundance',
-    'alpha': 'alpha_viscosity_numerical',
-    'Bxyz': 'magnetic_field',
-    'curlBxyz': 'magnetic_field_curl',
-    'deltavxyz': 'differential_velocity',
-    'divB': 'magnetic_field_divergence',
-    'divv': 'velocity_divergence',
-    'dt': 'timestep',
-    'dustfrac': 'dust_fraction',
-    'eta_AD': 'ambipolar_diffusion_coefficient',
-    'eta_HE': 'hall_effect_coefficient',
-    'eta_OR': 'ohmic_resistivity_coefficient',
-    'gr_density': 'density_general_relativity',
-    'gr_entropy': 'entropy_general_relativity',
-    'gr_momentum': 'momentum_general_relativity',
-    'graindens': 'grain_density',
-    'grainsize': 'grain_size',
-    'h': 'smoothing_length',
-    'itype': 'type',
-    'luminosity': 'luminosity',
-    'ne_on_n': 'electron_fraction',
-    'poten': 'gravitational_potential',
-    'pressure': 'pressure',
-    'psi': 'magnetic_field_psi',
-    'St': 'stokes_number',
-    'T': 'temperature',
-    'temperature_dust': 'temperature_dust',
-    'tstop': 'stopping_time',
-    'u': 'internal_energy',
-    'vrel_on_vfrag': 'relative_on_fragmentation_velocity',
-    'vxyz': 'velocity',
-    'xyz': 'position',
-}
 
-# Dictionary to convert Phantom HDF5 sinks array names to Plonk names
-sink_array_name_map = {
-    'h': 'accretion_radius',
-    'hsoft': 'softening_radius',
-    'lum': 'luminosity',
-    'm': 'mass',
-    'maccreted': 'mass_accreted',
-    'mdotloss': 'mass_loss_rate',
-    'Reff': 'radius_effective',
-    'spinxyz': 'spin',
-    'Teff': 'temperature_effective',
-    'tlast': 'last_injection_time',
-    'vxyz': 'velocity',
-    'xyz': 'position',
-}
+def fileid(snap: Snap) -> str:
+    """Return Phantom 'fileid'.
+
+    This contains information on the type of snapshot and which physics
+    the simulation contains.
+
+    Parameters
+    ----------
+    snap
+        The Snap object.
+
+    Returns
+    -------
+    str
+        The file id.
+    """
+    return snap._file_pointer['header/fileident'][()].decode()
 
 
-def generate_snap_from_file(filename: Union[str, Path]) -> Snap:
+def generate_snap_from_file(
+    filename: Union[str, Path], config: Union[str, Path] = None
+) -> Snap:
     """Generate a Snap object from a Phantom HDF5 file.
 
     Parameters
     ----------
     filename
         The path to the file.
+    config : optional
+        The path to a Plonk config.toml file.
 
     Returns
     -------
@@ -101,13 +74,19 @@ def generate_snap_from_file(filename: Union[str, Path]) -> Snap:
     file_handle = h5py.File(file_path, mode='r')
     snap._file_pointer = file_handle
 
-    # REQUIRED: Set snap._properties, snap.code_units, and snap._array_code_units.
+    # REQUIRED: Set snap._properties, snap._code_units, and snap._array_code_units.
     header = {key: val[()] for key, val in file_handle['header'].items()}
-    snap._properties, units = header_to_properties(header)
-    snap._array_code_units = generate_array_units_dict(units)
-    snap.code_units = {
-        key: units[key]
-        for key in ['length', 'time', 'mass', 'temperature', 'magnetic_field']
+    snap._properties, snap._code_units = header_to_properties(header)
+    snap._array_code_units = generate_array_code_units(snap._code_units)
+
+    # OPTIONAL: Set snap._units.
+    snap._default_units = array_units(config=config)
+
+    # OPTIONAL: Set snap._name_map.
+    conf = load_config(filename=config)
+    snap._name_map = {
+        'particles': conf['phantom']['particles']['namemap'],
+        'sinks': conf['phantom']['sinks']['namemap'],
     }
 
     # REQUIRED: Set snap._array_registry dictionary.
@@ -119,7 +98,7 @@ def generate_snap_from_file(filename: Union[str, Path]) -> Snap:
     ndustlarge = header['ndustlarge']
     array_registry = populate_particle_array_registry(
         arrays=arrays,
-        name_map=particle_array_name_map,
+        name_map=snap._name_map['particles'],
         ndustsmall=ndustsmall,
         ndustlarge=ndustlarge,
     )
@@ -132,10 +111,13 @@ def generate_snap_from_file(filename: Union[str, Path]) -> Snap:
     if header['nptmass'] > 0:
         sinks = list(file_handle['sinks'])
         sink_registry = populate_sink_array_registry(
-            sinks=sinks, name_map=sink_array_name_map
+            sinks=sinks, name_map=snap._name_map['sinks']
         )
         snap._sink_registry.update(sink_registry)
 
+    # OPTIONAL: Add aliases
+    add_aliases(snap)
+
     # OPTIONAL: Make extra derived quantities available.
     extra_quantities(snap)
 
@@ -167,7 +149,15 @@ def header_to_properties(header: dict):
         * plonk_units('g ** (1/2) / cm ** (1/2) / s')
         * np.sqrt(plonk_units.magnetic_constant / (4 * np.pi))
     ).to_base_units()
-    units = generate_code_units_dict(length, mass, time, temperature, magnetic_field)
+    current = (mass / time ** 2 / magnetic_field).to_base_units()
+
+    units = {
+        'length': length,
+        'time': time,
+        'mass': mass,
+        'temperature': temperature,
+        'current': current,
+    }
 
     prop = dict()
 
@@ -201,7 +191,9 @@ def header_to_properties(header: dict):
     n_dust = ndustsmall + ndustlarge
     if n_dust > 0:
         prop['grain_size'] = header['grainsize'][:n_dust] * units['length']
-        prop['grain_density'] = header['graindens'][:n_dust] * units['density']
+        prop['grain_density'] = (
+            header['graindens'][:n_dust] * units['mass'] / units['length'] ** 3
+        )
 
     return prop, units
 
@@ -327,19 +319,19 @@ def get_dataset(dataset: str, group: str) -> Callable:
 
     def func(snap: Snap) -> Quantity:
         array = snap._file_pointer[f'{group}/{dataset}'][()]
-        if dataset in particle_array_name_map:
-            unit = snap._array_code_units[particle_array_name_map[dataset]]
-        elif dataset in sink_array_name_map:
-            unit = snap._array_code_units[sink_array_name_map[dataset]]
+        name_map = snap._name_map[group]
+        if dataset in name_map:
+            name = name_map[dataset]
         else:
-            try:
-                unit = snap._array_code_units[dataset]
-            except KeyError:
-                logger.error(
-                    f'Cannot get unit of dataset "{group}/{dataset}" - '
-                    'assuming dimensionless'
-                )
-                unit = plonk_units('dimensionless')
+            name = dataset
+        try:
+            unit = snap._array_code_units[name]
+        except KeyError:
+            logger.error(
+                f'Cannot get unit of dataset "{group}/{dataset}" - '
+                'assuming dimensionless'
+            )
+            unit = plonk_units('dimensionless')
         return array * unit
 
     return func
