@@ -80,45 +80,14 @@ class Snap:
     ...     return plonk.analysis.particles.radial_distance(snap)
     """
 
-    _array_aliases = {
-        'B': 'magnetic_field',
-        'c_s': 'sound_speed',
-        'dt': 'timestep',
-        'e': 'eccentricity',
-        'eps': 'dust_fraction',
-        'h': 'smoothing_length',
-        'j': 'specific_angular_momentum',
-        'L': 'angular_momentum',
-        'm': 'mass',
-        'omega_k': 'keplerian_frequency',
-        'p': 'momentum',
-        'P': 'pressure',
-        'pos': 'position',
-        'phi': 'azimuthal_angle',
-        'R': 'radius_cylindrical',
-        'r': 'radius_spherical',
-        'rho': 'density',
-        'rho_d': 'dust_density',
-        'rho_g': 'gas_density',
-        'St': 'stokes_number',
-        'T': 'temperature',
-        'theta': 'polar_angle',
-        't_s': 'stopping_time',
-        'u': 'internal_energy',
-        'v': 'velocity',
-        'v_R': 'velocity_radial_cylindrical',
-        'v_r': 'velocity_radial_spherical',
-        'v_phi': 'angular_velocity',
-        'velocity_R': 'radial_velocity_cylindrical',
-        'velocity_r': 'radial_velocity_spherical',
-        'velocity_phi': 'angular_velocity',
-    }
+    _array_aliases: Dict[str, str] = {}
 
     _vector_arrays = {
         'magnetic_field',
         'position',
         'spin',
         'velocity',
+        'vorticity',
     }
 
     _dust_arrays = {
@@ -136,31 +105,21 @@ class Snap:
         'bulge': 6,
     }
 
-    @staticmethod
-    def add_alias(name: str, alias: str) -> None:
-        """Add alias to array.
-
-        Parameters
-        ----------
-        name
-            The name of the array.
-        alias
-            The alias to reference the array.
-        """
-        Snap._array_aliases[alias] = name
-
     def __init__(self):
 
         self.data_source = None
         self.file_path = None
-        self.code_units = {}
+        self._code_units = {}
+        self._default_units = {}
         self._properties = {}
         self._array_code_units = {}
         self._array_registry: Dict[str, Callable] = {}
         self._sink_registry: Dict[str, Callable] = {}
+        self._name_map = {}
         self._cache_arrays = True
         self._arrays = {}
-        self._sinks = {}
+        self._sink_arrays = {}
+        self._sinks = None
         self._file_pointer = None
         self._num_particles = -1
         self._num_particles_of_type = -1
@@ -215,6 +174,18 @@ class Snap:
 
         return _add_array
 
+    def add_alias(self, name: str, alias: str) -> None:
+        """Add alias to array.
+
+        Parameters
+        ----------
+        name
+            The name of the array.
+        alias
+            The alias to reference the array.
+        """
+        self._array_aliases[alias] = name
+
     def add_unit(self, name: str, unit: str) -> Snap:
         """Add missing code unit to array.
 
@@ -373,10 +344,47 @@ class Snap:
         """Snap properties."""
         return {key: self._properties[key] for key in sorted(self._properties.keys())}
 
+    @property
+    def default_units(self) -> Dict[str, Any]:
+        """Snap default units."""
+        return {
+            key: self._default_units[key] for key in sorted(self._default_units.keys())
+        }
+
+    def set_units(self, **kwargs) -> Snap:
+        """Set default unit for arrays.
+
+        Parameters
+        ----------
+        kwargs
+            Keyword arguments with keys as the array name, e.g.
+            'pressure', and with values as the unit as a string, e.g.
+            'pascal'.
+
+        Examples
+        --------
+        Set multiple default units.
+
+        >>> snap.set_units(pressure='pascal', density='g/cm^3')
+        """
+        for key, val in kwargs.items():
+            if key not in self.default_units:
+                logger.info(f'adding array {key} to default_units dict')
+            self._default_units[key] = val
+
+        return self
+
+    @property
+    def code_units(self) -> Dict[str, Any]:
+        """Snap code units."""
+        return {key: self._code_units[key] for key in sorted(self._code_units.keys())}
+
     @property
     def sinks(self) -> Sinks:
         """Sink particle arrays."""
-        return Sinks(self)
+        if self._sinks is None:
+            self._sinks = Sinks(self)
+        return self._sinks
 
     @property
     def num_particles(self) -> int:
@@ -459,8 +467,9 @@ class Snap:
         if any((arrays, rotation, translation)):
             for arr in self.loaded_arrays():
                 del self._arrays[arr]
-            for arr in self.sinks.loaded_arrays():
-                del self._sinks[arr]
+            if self.num_sinks > 0:
+                for arr in self.sinks.loaded_arrays():
+                    del self._sink_arrays[arr]
         else:
             logger.warning('Select something to reset')
 
@@ -519,14 +528,15 @@ class Snap:
             if arr in self.loaded_arrays():
                 array_m, array_u = self._arrays[arr].magnitude, self._arrays[arr].units
                 self._arrays[arr] = _rotation.apply(array_m) * array_u
-            if arr in self.sinks.loaded_arrays():
-                array_m, array_u = self._sinks[arr].magnitude, self._sinks[arr].units
-                self._sinks[arr] = _rotation.apply(array_m) * array_u
+            if self.num_sinks > 0 and arr in self.sinks.loaded_arrays():
+                array_m = self._sink_arrays[arr].magnitude
+                array_u = self._sink_arrays[arr].units
+                self._sink_arrays[arr] = _rotation.apply(array_m) * array_u
         for arr in self._vector_arrays:
             if arr in self.loaded_arrays():
                 del self._arrays[arr]
-            if arr in self.sinks.loaded_arrays():
-                del self._sinks[arr]
+            if self.num_sinks > 0 and arr in self.sinks.loaded_arrays():
+                del self._sink_arrays[arr]
 
         if self.rotation is None:
             self.rotation = _rotation
@@ -574,8 +584,8 @@ class Snap:
             translation *= plonk_units(unit)
         if 'position' in self.loaded_arrays():
             self._arrays['position'] += translation
-        if 'position' in self.sinks.loaded_arrays():
-            self._sinks['position'] += translation
+        if self.num_sinks > 0 and 'position' in self.sinks.loaded_arrays():
+            self._sink_arrays['position'] += translation
 
         if self.translation is None:
             self.translation = translation
@@ -625,55 +635,49 @@ class Snap:
             ]
         return np.flatnonzero(self['type'] == self.particle_type[particle_type])
 
-    def subsnaps_by_type(
-        self, split_subtypes: bool = True
+    def subsnaps_as_dict(
+        self, squeeze: bool = False
     ) -> Dict[str, Union[SubSnap, List[SubSnap]]]:
         """Return particle-type subsnaps as a dict of SubSnaps.
 
         Parameters
         ----------
-        split_subtypes : optional
-            If True, split particle types into subtypes.
+        squeeze : optional
+            Squeeze sub-types. For each key, if False and the particle
+            family has sub-types then return a list of SubSnaps of each
+            sub-type. Otherwise return a SubSnap with all particle of
+            that type.
 
         Returns
         -------
         Dict
             A dict of all SubSnaps.
         """
-        if not split_subtypes:
-            raise NotImplementedError('Combined sub-types not implemented yet')
-        subsnaps_by_type: Dict[str, Any] = dict()
+        subsnaps_as_dict: Dict[str, Any] = dict()
         for key in self.particle_type:
             try:
-                subsnap = self[key]
-                if isinstance(subsnap, list):
-                    if len(subsnap[0]) > 0:
-                        subsnaps_by_type[key] = self[key]
-                else:
-                    if len(subsnap) > 0:
-                        subsnaps_by_type[key] = self[key]
+                subsnaps_as_dict[key] = self.family(key, squeeze=squeeze)
             except ValueError:
-                pass
+                continue
 
-        return subsnaps_by_type
+        return subsnaps_as_dict
 
-    def subsnaps_as_list(self, split_subtypes: bool = True) -> List[SubSnap]:
+    def subsnaps_as_list(self, squeeze: bool = False) -> List[SubSnap]:
         """Return particle-type subsnaps as a list of SubSnaps.
 
         Parameters
         ----------
-        split_subtypes : optional
-            If True, split particle types into subtypes.
+        squeeze : optional
+            Squeeze sub-types. If True then each particle sub-type of
+            the same type will be treated the same.
 
         Returns
         -------
         List[SubSnap]
             A list of all SubSnaps.
         """
-        if not split_subtypes:
-            raise NotImplementedError('Combined sub-types not implemented yet')
         subsnaps: List[SubSnap] = list()
-        for val in self.subsnaps_by_type().values():
+        for val in self.subsnaps_as_dict(squeeze=squeeze).values():
             if isinstance(val, list):
                 subsnaps = subsnaps + val
             else:
@@ -840,7 +844,7 @@ class Snap:
             filename = f'{self.file_path.stem}_extra.h5'
         f = h5py.File(filename, mode='r')
         for array in f:
-            self[array] = f[array][()] * plonk_units['dimensionless']
+            self[array] = f[array][()] * plonk_units('dimensionless')
         f.close()
 
         return self
@@ -876,7 +880,7 @@ class Snap:
                     arr: Quantity = self[column]
                     _units.append(arr.units)
                 except AttributeError:
-                    _units.append(plonk_units['dimensionless'])
+                    _units.append(plonk_units('dimensionless'))
         else:
             _units = list()
             for unit in units:
@@ -928,7 +932,11 @@ class Snap:
         if name in self.particle_type:
             ind = self.particle_indices(particle_type=name, squeeze=squeeze)
             if isinstance(ind, list):
+                if len(ind[0]) == 0:
+                    raise ValueError('Family has no particles')
                 return [SubSnap(self, _ind) for _ind in ind]
+            if len(ind) == 0:
+                raise ValueError('Family has no particles')
             return SubSnap(self, ind)
         raise ValueError('Family not available')
 
@@ -995,7 +1003,7 @@ class Snap:
         ndarray
             The array on the particles in code units.
         """
-        return (self[name] / self.array_code_unit(name)).magnitude
+        return (self[name] / self.array_code_unit(name)).to_reduced_units().magnitude
 
     def base_array_name(self, name: str) -> str:
         """Get the base array name from a string.
@@ -1016,13 +1024,13 @@ class Snap:
         """
         if name in self.available_arrays():
             return name
-        if name in self.sinks.available_arrays():
+        if self.num_sinks > 0 and name in self.sinks.available_arrays():
             return name
         if name in self._array_aliases:
             return self._array_aliases[name]
         if name in self._arrays:
             return name
-        if name in self._sinks:
+        if name in self._sink_arrays:
             return name
 
         name_root = '_'.join(name.split('_')[:-1])
@@ -1040,15 +1048,13 @@ class Snap:
         raise ValueError('Unknown array')
 
     def _array_suffix(self, name: str) -> str:
-        if (
-            name in self.available_arrays()
-            or name in self.sinks.available_arrays()
-            or name in self._array_aliases
-        ):
+        if name in self.available_arrays() or name in self._array_aliases:
+            return ''
+        if self.num_sinks > 0 and name in self.sinks.available_arrays():
             return ''
         if name in self._arrays:
             return ''
-        if name in self._sinks:
+        if name in self._sink_arrays:
             return ''
 
         name_root = '_'.join(name.split('_')[:-1])
@@ -1105,16 +1111,23 @@ class Snap:
     def _get_array(self, name: str, sinks: bool = False) -> Quantity:
         """Get an array by name."""
         if sinks:
-            array_dict = self._sinks
+            array_dict = self._sink_arrays
         else:
             array_dict = self._arrays
         if name in array_dict:
+            if name in self.default_units:
+                return array_dict[name].to(self.default_units[name])
             return array_dict[name]
         if name in self._array_registry or name in self._sink_registry:
-            array = self._get_array_from_registry(name, sinks)
+            if name in self.default_units:
+                array = self._get_array_from_registry(name, sinks).to(
+                    self.default_units[name]
+                )
+            else:
+                array = self._get_array_from_registry(name, sinks)
             if self.cache_arrays:
                 if sinks:
-                    self._sinks[name] = array
+                    self._sink_arrays[name] = array
                 else:
                     self._arrays[name] = array
             return array
@@ -1222,7 +1235,7 @@ class SubSnap(Snap):
 
         ind = _input_indices_array(inp=indices, max_slice=len(base))
         if len(ind) == 0:
-            raise ValueError('SubSnap has no particles')
+            logger.warning('SubSnap has no particles')
         self._indices = ind
 
         # Attributes different to Snap
@@ -1234,14 +1247,15 @@ class SubSnap(Snap):
         # Attributes same as Snap
         self.data_source = self.base.data_source
         self.file_path = self.base.file_path
-        self.code_units = self.base.code_units
+        self._code_units = self.base._code_units
+        self._default_units = self.base._default_units
         self._properties = self.base._properties
         self._array_code_units = self.base._array_code_units
         self._array_registry = self.base._array_registry
         self._sink_registry = self.base._sink_registry
         self._cache_arrays = self.base._cache_arrays
         self._arrays = self.base._arrays
-        self._sinks = self.base._sinks
+        self._sink_arrays = self.base._sink_arrays
         self._file_pointer = self.base._file_pointer
         self.rotation = self.base.rotation
         self.translation = self.base.translation
@@ -1309,11 +1323,14 @@ class Sinks:
             indices = np.arange(base.num_sinks)
         ind = _input_indices_array(inp=indices, max_slice=base.num_sinks)
         if len(ind) == 0:
-            raise ValueError('Sinks has no particles')
+            logger.warning('Sinks has no particles')
         self._indices = ind
+        self._iter_index = 0
 
         # Attributes same as Snap
         self.file_path = self.base.file_path
+        self.base_array_name = self.base.base_array_name
+        self.default_units = self.base.default_units
 
     @property
     def indices(self) -> ndarray:
@@ -1348,7 +1365,7 @@ class Sinks:
         List
             A list of names of loaded sink arrays.
         """
-        return sorted(self.base._sinks.keys())
+        return sorted(self.base._sink_arrays.keys())
 
     def array(self, name: str) -> Quantity:
         """Get an array.
@@ -1370,7 +1387,7 @@ class Sinks:
             raise ValueError('"item" must be an array with units, i.e. a Pint Quantity')
         if item.shape[0] != len(self):
             raise ValueError('Length of array does not match particle number')
-        self.base._sinks[name] = item
+        self.base._sink_arrays[name] = item
 
     def __getitem__(self, inp):
         """Return an array or subset."""
@@ -1378,7 +1395,8 @@ class Sinks:
             return self.array(name=inp)
         ind = _input_indices_array(inp=inp, max_slice=len(self))
         if ind is not None:
-            return Sinks(self.base, ind)
+            _ind = self.indices[ind]
+            return Sinks(self.base, _ind)
         raise ValueError('Cannot determine item to return')
 
     def __len__(self) -> int:
@@ -1393,6 +1411,30 @@ class Sinks:
         """Dunder str method."""
         return f'<plonk.Sinks "{self.base.file_path.name}">'
 
+    def __add__(self, other):
+        """Add Sinks."""
+        indices = list(self.indices) + list(other.indices)
+        return self.base.sinks[indices]
+
+    def __eq__(self, other):
+        """Compare Sinks."""
+        same_base = self.base == other.base
+        same_indices = np.all(self.indices == other.indices)
+        return same_base and same_indices
+
+    def __iter__(self):
+        """Sinks iterator."""
+        return self
+
+    def __next__(self):
+        """Sinks iterator."""
+        if self._iter_index > len(self) - 1:
+            self._iter_index = 0
+            raise StopIteration()
+        index = self._iter_index
+        self._iter_index += 1
+        return self[index]
+
     def _ipython_key_completions_(self) -> List[str]:
         """Tab completion for IPython __getitem__ method."""
         return self.available_arrays(verbose=True)
@@ -1418,9 +1460,8 @@ def _input_indices_array(
         if np.issubdtype(np.int, inp.dtype):
             return inp
     if isinstance(inp, (list, tuple)):
-        if isinstance(inp[0], int):
-            return np.array(inp)
-    if isinstance(inp, int):
+        return np.array(inp)
+    if isinstance(inp, (int, np.integer)):
         return np.array([inp])
     if isinstance(inp, slice):
         i1 = inp.start if inp.start is not None else 0
